class Pog
  
  # options expected to include client_id, client_secret, and redirect_uri
  def initialize(options)
    @client_id = options[:client_id]
    @client_secret = options[:client_secret]
    @redirect_uri = options[:redirect_uri]
  end

  def has_permission?
    self.permission
  end

  def access_token_expired?
    self.access_token_expiration_time < Time.now
  end

  def retrieve_auth(options)
    scope = create_uri_from_scopes(options)
    @client = Signet::OAuth2::Client.new(
      :authorization_uri => 'https://accounts.google.com/o/oauth2/auth',
      :token_endpoint_uri => 'https://accounts.google.com/o/oauth2/token',
      :client_id => @client_id,
      :client_secret => @client_secret,
      :scope => scope,
      :redirect_uri => @redirect_uri
    )
    @client.authorization_uri.to_s
  end

  private

  def create_uri_from_scope(options)
    "https://www.googleapis.com/auth/#{options[:api]}.readonly" if options[:scope] == "read"
  end

  # Takes an auth code provided when user consents to grant
  # authorization. 
  def request_access_token(auth_code)
    http = Net::HTTP.new('accounts.google.com', 443)
    path = '/o/oauth2/token'
    http.use_ssl = true
    data = "code=#{auth_code}&client_id=#{@client_id}&client_secret=#{@client_secret}&redirect_uri=#{@redirect_uri}&grant_type=authorization_code"
    response = http.post(path, data)
    values = JSON.parse(response.body)
  end

  def evaluate_response_code(code)
    code = code.to_i
    if code === 200
      return true
    else
      revoke_permission if code === 403
      return false
    end
  end

  def grant_permission
    self.has_permission = true
    self.save!
  end

  def revoke_permission
    self.has_permission = false
    self.save!
  end

end